/*
 * Software for testing human discrimination of images composed by block
 * patterns as described in the paper "The Number of All Possible Meaningful
 * or Discernible Pictures" by Theo Pavlidis
 *
 * COPYRIGHT NOTICE, DISCLAIMER, and LICENSE:

 * Copyright 2009(c) Theo Pavlidis

 * This is 1.1 version (April 2009) of the library and the USE AT YOUR OWN RISK
 * warning remains particularly applicable.
 *
 * Permission is hereby granted to use, copy, modify, and distribute this
 * source code, or portions hereof, for any purpose, without fee, subject
 * to the following restrictions:
 *
 * 1. The origin of this source code must not be misrepresented.
 *
 * 2. Altered versions must be plainly marked as such and
 * must not be misrepresented as being the original source.
 *
 * 3. This Copyright notice may not be removed or altered from
 *    any source or altered source distribution.

 * Comments are welcome. Send to t.pavlidis@ieee.org

*/
#pragma once

//	The structure matpair (given below) is the core of the program.
//	The rest of the code depends on the particular system used and
//  it deals with user interaction and display code.

// Stucture for statistics
typedef struct _TimeStats {
	double total;
	int n;
} TimeStats;

//	This strcture contains all the code for creating the random patterns
typedef struct _matpair {
	int K, nC, boxsz;	// given
	int *mat, flip, nvalue;	// computed
	BOOL fill() {
		if(mat != NULL) delete[] mat;
		mat = new int[K*K];	if(mat==NULL) return FALSE;
		srand( (unsigned)time( NULL ) );
		for(int i=0; i<K*K; i++) mat[i] = rand()%nC;
		flip = rand()%(K*K);
		if(nC==2) nvalue = 1 - mat[flip];
		else {
			int z = rand()%nC;
			while(z==mat[flip]) z = rand()%nC;
			nvalue = z;
		}
		return TRUE;
	}
} matpair;

#define H_OFF 50
#define W_OFF 50

// CPixWindow

class CPixWindow : public CWnd
{
	DECLARE_DYNAMIC(CPixWindow)

public:
	CPixWindow();
	virtual ~CPixWindow();

	int PopWindow(int x0, int y0, int k, int nC, int bxsz, BOOL useIcons);
	void InitStats() { 	m_stat.n = 0;	m_stat.total = 0; }
	void QueryStats() {
		CString tmp;
		if(m_stat.n < 1) tmp = "No Stats";
		else tmp.Format(
			"K=%d, NC=%d %s\nAVG TIME = %5.2f (%d tests)",
			m_Mat.K, m_Mat.nC, m_UseIcons==TRUE? "small pictures" : "colored squares",
			m_stat.total/m_stat.n, m_stat.n);
		::AfxMessageBox(tmp);
	}

protected:
	//	where all the important stuff is
	matpair m_Mat;
	//	to display icons rather than boxes
	BOOL m_UseIcons;
	HICON m_IconList[4];
	//	user feedback
	int m_start_time;
	CString m_msg;
	CPoint m_point;
	TimeStats m_stat;

	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnPaint();
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
};


